// Library:       MQSeries
// Component:     C++ sample Programs
// Part:          IMQSPUT.CPP
//
// Description:   Sample C++ program that puts messages to a named
//                queue.
//
//                IMQSPUT has 2 parameters:
//                - the name of a queue (required)
//                - the name of a queue manager (optional)
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager.
//
//  Statement:     Licensed Materials - Property of IBM
//
//                 04L1773, 5765-B73
//                 04L1802, 5639-B42
//                 04L1788, 5765-B74
//                 04L1816, 5765-B75
//                 04L1830, 5639-B43
//                 (C) Copyright IBM Corp. 1994, 1998
//
//                U.S. Government Users Restricted Rights - use,
//                duplication or disclosure restricted by GSA ADP
//                Schedule Contract with IBM Corp.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <imqi.hpp> // MQSeries MQI

int main ( int argc, char * * argv ) {
  ImqQueueManager mgr;             /* Queue manager                 */
  ImqQueue queue;                  /* Queue                         */
  ImqMessage msg;                  /* Data message                  */
  int      buflen;                 /* buffer length                 */
  char     buffer[ 256 ];          /* message buffer                */

  printf( "Sample IMQSPUT start\n" );
  if ( argc < 2 ) {
    printf( "Required parameter missing - queue name\n" );
    exit( (int)99 );
  }

  /******************************************************************/
  /*                                                                */
  /*   Connect to queue manager                                     */
  /*                                                                */
  /******************************************************************/
  if ( argc > 2 ) {
    mgr.setName( argv[2] );
  }

  if ( ! mgr.connect( ) ) {

    /* stop if it failed */
    printf( "ImqQueueManager::connect ended with reason code %d\n",
            (int)mgr.reasonCode( ) );
    exit( (int)mgr.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Associate queue with queue manager.                          */
  /*                                                                */
  /******************************************************************/
  queue.setConnectionReference( mgr );

  /******************************************************************/
  /*                                                                */
  /*   Use parameter as the name of the target queue                */
  /*                                                                */
  /******************************************************************/
  queue.setName( argv[ 1 ] );
  printf( "target queue is %s\n", argv[ 1 ] );

  /******************************************************************/
  /*                                                                */
  /*   Open the target message queue for output                     */
  /*                                                                */
  /******************************************************************/
  queue.setOpenOptions( MQOO_OUTPUT /* open queue for output        */
    + MQOO_FAIL_IF_QUIESCING );     /* but not if MQM stopping      */
  queue.open( );

  /* report reason, if any; stop if failed      */
  if ( queue.reasonCode( ) ) {
    printf( "ImqQueue::open ended with reason code %d\n",
            (int)queue.reasonCode( ) );
  }

  if ( queue.completionCode( ) == MQCC_FAILED ) {
    printf( "unable to open queue for output\n" );
  }

  /******************************************************************/
  /*                                                                */
  /*  Read lines from the console and put them to the message queue */
  /*  Loop until end of input, or there is a failure                */
  /*                                                                */
  /******************************************************************/
  msg.useEmptyBuffer( buffer, sizeof( buffer ) );
  msg.setFormat( MQFMT_STRING );      /* character string format    */

  while ( queue.completionCode( ) != MQCC_FAILED ) {

    if ( gets( buffer ) ) {           /* console read successful    */
      buflen = (int)strlen( buffer ); /* determine string length    */
    } else {
      buflen = 0;           /* treat EOF same as null line          */
    }

    /****************************************************************/
    /*                                                              */
    /*   Put each buffer to the message queue                       */
    /*                                                              */
    /****************************************************************/
    if ( buflen > 0 ) {
      msg.setMessageLength( buflen );
      if ( ! queue.put( msg ) ) {

        /* report reason, if any */
        printf( "ImqQueue::put ended with reason code %d\n",
                (int)queue.reasonCode( ) );
      }

    } else {
      /* quit loop when empty line is read */
      break ;
    }
  }

  /******************************************************************/
  /*                                                                */
  /*   Close the target queue (if it was opened)                    */
  /*                                                                */
  /******************************************************************/
  if ( ! queue.close( ) ) {

    /* report reason, if any     */
    printf( "ImqQueue::close ended with reason code %d\n",
            (int)queue.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Disconnect from MQM if not already connected (the            */
  /*   ImqQueueManager object handles this situation automatically) */
  /*                                                                */
  /******************************************************************/
  if ( ! mgr.disconnect( ) ) {

    /* report reason, if any     */
    printf( "ImqQueueManager::disconnect ended with reason code %d\n",
            (int)mgr.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /* END OF IMQSPUT                                                 */
  /*                                                                */
  /******************************************************************/
  printf( "Sample IMQSPUT end\n" );
  return( 0 );
}
